import {useMemo, useCallback, useContext, useEffect, useState, DependencyList} from 'react';
import { ProstoreSsrContext } from './ProstoreSsrContext';
import {CheckRequestResult, checkRequestState} from '@proscom/prostore/lib/storeUtils';
import { useConnectStore } from './useConnectStore';
import {useContextStore} from './ProstoreContext';
import {IRequestState, RequestStore} from '@proscom/prostore/lib/RequestStore';

export interface UseRequestStoreResult<State> {
  check: CheckRequestResult;
  state: State;
  load: () => void;
}

/**
 * Хук, подключающий компонент к стору и запрашивающий данные
 * в соответствии с variables
 */
export function useRequestStore<Vars, Data, Options>(storeOrName: string | RequestStore<Vars, Data, Options>, variables: Vars, options: Partial<Options> = null): UseRequestStoreResult<IRequestState<Vars, Data>> {
  // Подключаем стор из контекста
  const store: RequestStore<Vars, Data, Options> = useContextStore(storeOrName) as RequestStore<Vars, Data, Options>;

  // Подключение к контексту, который хранит все результаты
  const ssrContext = useContext(ProstoreSsrContext);

  // Локальная копия состояния стора
  const [state, setState] = useState(() => {
    // Если в контексте есть значение, используем его
    return store.getValueFromContext(ssrContext);
  });
  useConnectStore(store, setState);

  // Проверка, нужно ли запрашивать данные
  const queryCheck = checkRequestState(state, variables);

  // Мемоизированный колбек для загрузки данных
  const loadData = useCallback(() => {
    return store.loadData(variables, {
      ssr: ssrContext && ssrContext.isServer,
      ...options
    }).then(() => {
      store.setContextValue(ssrContext);
    });
  }, [store, variables, options]);

  // Запрос данных, если он нужен
  useEffect(() => {
    if (queryCheck.request) {
      // Вероятность ошибки в этом месте мала, т.к.
      // основные ошибки обрабатываются в RequestStore
      // Поэтому мы ее просто залогируем
      loadData().catch((e) => console.error(e));
    }
  }, [queryCheck, loadData]);

  // Если мы сейчас в режиме серверного рендеринга, то запрашиваем данные
  if (ssrContext && ssrContext.isServer && queryCheck.request && store.ssrId) {
    ssrContext.registerPromise(loadData());
  }

  // Возвращаем то, что может быть полезно в самом компоненте
  return { check: queryCheck, state, load: loadData };
}

export function useRequestStoreVars<Vars, Data>(storeOrName: string | RequestStore<Vars, Data, any>, variableCreator: () => Vars, args: DependencyList) {
  const variables = useMemo(variableCreator, args);
  return useRequestStore(storeOrName, variables);
}
